/******************************************************************************
 * file: DoubleLinkedList.java
 * 
 * description: provides a class to create double linked lists and methods
 * to perform typical operations on these type of data set
 *****************************************************************************/


import java.util.Iterator;
class Node_DLLS {

  public Node_DLLS(Object o, Node_DLLS prev, Node_DLLS next) {
    this.content = o;
    this.nextNode = next;
    this.prevNode = prev;
  }

  /* returns the content of this node */
  public Object getContent() {
    return this.content;
  }


  /* sets the content of this node to o */
  public void setContent(Object o) {
    this.content = o;
  }


  /* returns the predecessor of this node */
  public Node_DLLS getPreviousNode() {
    return this.prevNode;
  }


  /* sets the predecessor of this node to p */
  public void setPreviousNode(Node_DLLS p) {
    this.prevNode = p;
  }


  /* returns the successor of this node */
  public Node_DLLS getNextNode() {
    return this.nextNode;
  }


  /* sets the successor of this node to n */
  public void setNextNode(Node_DLLS n) {
    this.nextNode  = n;
  }


  /* returns the node's content as a string */
  public String toString() {
    return (String) this.content;
  }
  private Object content;
  private Node_DLLS nextNode;
  private Node_DLLS prevNode;

}


/* a double linked list */
public class DoubleLinkedListSimple {


  /* variables */
  protected Node_DLLS head;
  protected Node_DLLS tail;

  /* constructor */
  public DoubleLinkedListSimple() {
    this.head = null;
    this.tail = null;
  }


  /* returns true if this list is empty, otherwise false */
  public boolean isEmpty() {
    return (this.head == null);
  }

  public void insertHead(Object o) {
    if (this.isEmpty())
      this.head = this.tail = new Node_DLLS(o, null, null);
    else {
      this.head.setPreviousNode(new Node_DLLS(o, null, this.head));
      this.head = this.head.getPreviousNode();
    }
  }

  public void insertTail(Object o) {
    if (this.isEmpty())
      this.insertHead(o);
    else {
      this.tail.setNextNode(new Node_DLLS(o, this.tail, null));
      this.tail = this.tail.getNextNode();
    }
  }

  public void removeHead(){
    this.removeNode(this.head);
  }

  public void removeTail(){
    this.removeNode(this.tail);
  }

  public void removeNode(Node_DLLS n){
    Node_DLLS nextNode = n.getNextNode();
    Node_DLLS prevNode = n.getPreviousNode();
    if (this.head == n)
      this.head = nextNode;
    if (this.tail == n)
      this.tail = prevNode;
    if (prevNode != null)
      prevNode.setNextNode(nextNode);
    if (nextNode != null)
      nextNode.setPreviousNode(prevNode);
  }

  public void reverse(){
    Node_DLLS tmp = this.head;
    while (tmp != null){
      // swap prev and next
      Node_DLLS next  = tmp.getNextNode();
      tmp.setNextNode(tmp.getPreviousNode());
      tmp.setPreviousNode(next);
      tmp = next;
    }
    // swap head and tail
    tmp = this.head;
    this.head = this.tail;
    this.tail = tmp;
  }


  /* returns the list's content as a string object */
  public String toString() {
    String str = "[";
    Node_DLLS tmp = this.head;
    while (tmp != null) {
      str += tmp.getContent().toString();
      tmp = tmp.getNextNode();
      if (tmp != null)
        str += ", ";
    }
    return str+"]";
  }

}







