/******************************************************************************
 * file: SingleLinkedList.java
 * 
 * description: provides a class to create single linked lists and methods
 * to perform typical operations on these type of data set
 *****************************************************************************/


import java.util.Iterator;

/* implements the interface Iterator */
class SingleLinkedListIterator implements Iterator {

  public SingleLinkedListIterator(Node_SLL node) {
    this.node = node;
  }

  public boolean hasNext() {
    return (this.node != null);
  }

  public Object next() {
    Object o = node.getContent();
    this.node  = this.node.getNextNode();
    return o;
  }
  public void remove() {
    //this method has to be implemented!
  }

  private Node_SLL node;
}


/* a node of a single linked list has a content and a successor */  
class Node_SLL {


  /* constructor */
  public Node_SLL (Object o, Node_SLL n) {
    this.content = o;
    this.nextNode = n;
  }


  /* returns the content of this node */
  public Object getContent() {
    return this.content;
  }


  /* sets the content of this node to o */
  public void setContent(Object o) {
    this.content = o;
  }


  /* returns the successor of this node */
  public Node_SLL getNextNode() {
    return this.nextNode;
  }


  /* sets the successor of this node to n */
  public void setNextNode(Node_SLL n) {
    this.nextNode = n;
  }


  /* returns the node's content as a string */
  public String toString() {
    return this.content.toString();
  }

  /* variables */
  private Object content;
  private Node_SLL nextNode;



}


/* a single linked list */
public class SingleLinkedList {


  /* constructor */
  public SingleLinkedList() {
    this.head = null;
  }


  /* returns true if this list is empty, otherwise false */
  public boolean isEmpty() {
    return (this.head == null);
  }


  /* returns the reference of the node, that contains Object o */
  /* if this list contains Object o, otherwise null */
  public Node_SLL search1(Object o) {
    Node_SLL current = this.head;
    while (current != null) {
      if (current.getContent().equals(o)) {
        return current;
      }
      current = current.getNextNode(); 
    }
    return null;
  }

  /* returns the reference of the Object o */
  /* if this list contains Object o, otherwise null */

  public boolean contains(Object o, SingleLinkedList l) {
    Iterator i = l.iterator();
    while (i.hasNext()) {
      Object content = i.next();
      if (content.equals(o))
        return true;
    }
    return false;
  }

  public Node_SLL searchNode(Object o) {
    Node_SLL n = this.head;
    while (n != null && !n.getContent().equals(o))
      n = n.getNextNode();
    return n;
  }


  public void insertAfterNode(Object o, Node_SLL n) {
    n.setNextNode(new Node_SLL(o, n.getNextNode()));
    System.out.println("insertingAfter " + o);
  }

  /* inserts a Node that contains Object o at the head*/
  public void insertFirst(Object o) {
    this.head = new Node_SLL(o, this.head);
    System.out.println("insertingFirst " + o);
  }


  /* inserts a Node that contains Object o at the tail*/
  public void insertLast(Object o) {
    if (this.isEmpty()) { /* list is empty */
      this.insertFirst(o);
    }
    else {
      Node_SLL tmp = this.head;
      while (tmp.getNextNode() != null)
        tmp = tmp.getNextNode();
      tmp.setNextNode(new Node_SLL(o, null));
      System.out.println("insertingLast " + o);
    }
  }



  /* removes the first Node */
  void removeFirstNode() {
    if (! this.isEmpty()){
      System.out.println("removing first node");
      this.head = this.head.getNextNode();
    }
  }

  /* removes the next Node after Node */
  private void removeNextNode(Node_SLL node) {
    if (node.getNextNode() != null){
      System.out.println("removing "
          + node.getNextNode().getContent().toString());
      node.setNextNode(node.getNextNode().getNextNode());
    }
  }



  /* removes the first Node that contains Object o */
  public void remove(Object o) {
    Node_SLL current = this.head;
    Node_SLL predecessor = null;
    while (current != null) {
      if (current.getContent().equals(o)) {
        if (predecessor != null) {
          predecessor.setNextNode(current.getNextNode());
        } else {
          this.head = current.getNextNode();
        }
        System.out.println("removing " + o);
        break;
      }
      predecessor = current;
      current = current.getNextNode(); 
    }
  }


  private void reverseRecursive(Node_SLL node, Node_SLL pred){
    Node_SLL next = node.getNextNode();
    if (next == null)
      this.head = node;
    else {
      reverseRecursive(next, node);
    }
    node.setNextNode(pred);
  }

  /* reverses the list */
  public void reverseList() {
    if (!this.isEmpty()){
      reverseRecursive(this.head, null);
    }
    System.out.println("reversing list");
  }


  /* returns the list's content as a string object */
  public String toString() {
    String str = "[";
    Node_SLL tmp = this.head;
    while (tmp != null) {
      str += tmp.getContent().toString();
      tmp = tmp.getNextNode();
      if (tmp != null)
        str += ", ";
    }
    return str+"]";
  }

  public Iterator iterator(){
    return new SingleLinkedListIterator(this.head);
  }

  /* variables */
  private Node_SLL head;
}








